from decouple import AutoConfig, Csv

config = AutoConfig()


# Github Config
GITHUB_DEFAULT_PAGE_SIZE = config("GITHUB_DEFAULT_PAGE_SIZE", cast=int, default=100)
GITHUB_ORG = config("GITHUB_ORG", cast=str)
GITHUB_REPOSITORIES = config("GITHUB_REPOSITORIES", cast=Csv())
GITHUB_TOKEN = config("GITHUB_TOKEN", cast=str)
GITHUB_URL = config("GITHUB_URL", cast=str, default="https://api.github.com")
GITHUB_USER = config("GITHUB_USER", cast=str)



import click
from rich.console import Console

from deps.pipenv.view import DependenciesView

from ..version import __version__

CONTEXT_SETTINGS = dict(help_option_names=["-h", "--help"])


@click.group(context_settings=CONTEXT_SETTINGS)
@click.version_option(__version__, "-v", "--version", message="version %(version)s")
def cli() -> None:
    """Deps - A terminal UI Dashboard for monitoring code review requests.\n

    For feature requests or bug reports: https://github.com/apoclyps/deps/issues
    """


@cli.command(help="Visualize code review requests as a Dashboard")
def dashboard() -> None:
    """
    Command:\n
        deps dashboard
    """
    console = Console()
    DependenciesView(console=console).render()


def main() -> None:
    """Entry point to CLI"""
    cli()



from dataclasses import dataclass
from typing import Any

from rich.console import Console
from rich.table import Table

from deps.config import GITHUB_REPOSITORIES
from deps.pipenv.resolver import DependenciesResolver


@dataclass
class DependenciesView:
    """Responsible for rendering a table of all dependencies grouped by service"""

    console: Console
    used_versions_by_service: dict
    resolver: DependenciesResolver

    def __init__(self, console: Console) -> None:
        """Initialize the view"""
        self.used_versions_by_service = {}
        self.console = console
        self.resolver = DependenciesResolver()

    def render(self) -> None:
        """Render the view"""
        for repo in GITHUB_REPOSITORIES:
            file_dependencies: str = self.resolver.retrieve_file(repo=repo)

            used_versions_by_service: dict = self.resolver.compare_package_versions(repo=repo, lines=file_dependencies)
            if used_versions_by_service:
                self.used_versions_by_service = self.used_versions_by_service | used_versions_by_service

        if used_versions_by_service:
            for service_name, dependencies in used_versions_by_service.items():
                table = self.render_service_dependencies(
                    service_name=service_name,
                    dependencies=dependencies,
                )
                self.console.print(table)
        else:
            self.console.print("No dependencies found")

    def render_service_dependencies(self, service_name: str, dependencies: list[dict[Any, Any]]) -> Table:
        """Render the service dependencies"""
        table: Table = Table(title=service_name)

        table.add_column("Dependency Name", style="white", no_wrap=True)
        table.add_column("Current Version", style="red")
        table.add_column("Available Version", justify="right", style="green")

        for dependency in dependencies:
            current_version: str = dependency["current_version"]
            available_version: str = dependency["available_version"]

            # filter out versions that are identical
            if current_version != available_version:
                table.add_row(
                    dependency["dependency_name"],
                    current_version,
                    available_version,
                )

        return table



from dataclasses import dataclass

import requests


@dataclass
class PackageCache:
    """A cache of the latest versions of packages"""

    cache: dict[str, str]

    def __init__(self) -> None:
        """Initialize the cache"""
        self.cache = {}

    def retrieve(self, name: str) -> None:
        """Retrieve the latest version of a package from PyPI"""
        response = requests.get(f"https://pypi.org/pypi/{name}/json")

        if available_version := response.json()["info"]["version"]:
            self.cache[name] = str(available_version)

    def get(self, name: str) -> str:
        """Get the latest version of a package from the cache"""
        if name not in self.cache:
            self.retrieve(name)

        return self.cache[name]


cache: PackageCache = PackageCache()



from dataclasses import dataclass
from re import match

from requests import get
from requests.auth import HTTPBasicAuth

from deps.config import GITHUB_ORG, GITHUB_TOKEN, GITHUB_USER
from deps.storage import cache

REGEX_AVAILABLE_LINE = r"^Available\sversions:\s(?P<latest>[^,]*)"
REGEX_HEADER = r"\[.*\]"
REGEX_PACKAGE_HEADER = r"\[(?:dev-)?packages\]"
REGEX_PYPI_LINE = r"^(?P<name>.*)\s*\=\s*\"(?:==)?(?P<version>.*)\"$"


@dataclass
class DependenciesResolver:
    """Resolves package versions from Pipfile against PyPI"""

    versions_by_service: dict
    auth = None

    def __init__(self) -> None:
        """Initialize the resolver"""
        self.versions_by_service = {}
        self.auth = HTTPBasicAuth(GITHUB_USER, GITHUB_TOKEN)

    def retrieve_file(self, repo: str) -> str:
        """Retrieve the Pipfile from GitHub"""
        urls = [
            f"https://raw.githubusercontent.com/{GITHUB_ORG}/{repo}/main/Pipfile",
            f"https://raw.githubusercontent.com/{GITHUB_ORG}/{repo}/master/Pipfile",
        ]

        for url in urls:
            response = get(url=url, auth=self.auth)

            if response.status_code != 404:
                return response.content.decode(encoding="UTF8")

        return ""

    def compare_package_versions(self, repo: str, lines: str) -> dict:
        """Compare the package versions"""
        reading_packages = False

        if not lines:
            return {}

        for line in lines.split("\n"):
            # Only worry about package sections
            if match(REGEX_HEADER, line):
                reading_packages = bool(match(REGEX_PACKAGE_HEADER, line))

            if not reading_packages:
                continue

            # Grab package name and pip version
            if pypi_match := match(REGEX_PYPI_LINE, line):
                name = pypi_match["name"].strip()
                version = pypi_match["version"].strip()

                if repo not in self.versions_by_service:
                    self.versions_by_service[repo] = []

                available_version: str = cache.get(name=name)
                self.versions_by_service[repo].append(
                    {
                        "dependency_name": name,
                        "current_version": version,
                        "available_version": available_version,
                    }
                )

        return self.versions_by_service
